@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
  *  Copyright (C) 1996-2002 Hybrid Development Team
- *  Copyright (C) 2002-2005 ircd-ratbox development team
+ *  Copyright (C) 2002-2012 ircd-ratbox development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: commio.c 27143 2010-11-04 19:47:38Z androsyn $
+ *  $Id: commio.c 27389 2012-04-15 21:36:30Z androsyn $
  */
 #include <libratbox_config.h>
 #include <ratbox_lib.h>
@@ -2174,7 +2174,7 @@ rb_send_fd_buf(rb_fde_t *xF, rb_fde_t **F, int count, void *data, size_t datasiz
 	char empty = '0';
 	char *buf;
 
-	memset(&msg, 0, sizeof(&msg));
+	memset(&msg, 0, sizeof(msg));
 	if(datasize == 0)
 	{
 		iov[0].iov_base = &empty;
