@@ -4,7 +4,7 @@
  *
  * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center 
  * Copyright (C) 1996-2002 Hybrid Development Team 
- * Copyright (C) 2002-2005 ircd-ratbox development team 
+ * Copyright (C) 2002-2012 ircd-ratbox development team 
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,21 +18,19 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: channel.c 23037 2006-09-09 15:21:56Z leeh $
+ *  $Id: channel.c 27371 2012-03-16 05:33:15Z dubkat $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "struct.h"
 #include "channel.h"
 #include "client.h"
-#include "common.h"
 #include "hash.h"
 #include "hook.h"
-#include "irc_string.h"
-#include "sprintf_irc.h"
+#include "match.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "s_serv.h"		/* captab */
@@ -41,23 +39,23 @@
 #include "whowas.h"
 #include "s_conf.h"		/* ConfigFileEntry, ConfigChannel */
 #include "s_newconf.h"
-#include "event.h"
-#include "memory.h"
-#include "balloc.h"
 #include "s_log.h"
 
 struct config_channel_entry ConfigChannel;
-dlink_list global_channel_list;
-static BlockHeap *channel_heap;
-static BlockHeap *ban_heap;
-static BlockHeap *topic_heap;
-static BlockHeap *member_heap;
+rb_dlink_list global_channel_list;
+static rb_bh *channel_heap;
+static rb_bh *ban_heap;
+static rb_bh *topic_heap;
+static rb_bh *member_heap;
+struct ev_entry *checksplit_ev;
 
-static int channel_capabs[] = { CAP_EX, CAP_IE, 
+static int channel_capabs[] = { CAP_EX, CAP_IE,
 #ifdef ENABLE_SERVICES
-				CAP_SERVICE,
+	CAP_SERVICE,
 #endif
-				CAP_TS6 };
+	CAP_TS6
+};
+
 #define NCHCAPS         (sizeof(channel_capabs)/sizeof(int))
 #define NCHCAP_COMBOS   (1 << NCHCAPS)
 
@@ -74,10 +72,10 @@ static void free_topic(struct Channel *chptr);
 void
 init_channels(void)
 {
-	channel_heap = BlockHeapCreate(sizeof(struct Channel), CHANNEL_HEAP_SIZE);
-	ban_heap = BlockHeapCreate(sizeof(struct Ban), BAN_HEAP_SIZE);
-	topic_heap = BlockHeapCreate(TOPICLEN + 1 + USERHOST_REPLYLEN, TOPIC_HEAP_SIZE);
-	member_heap = BlockHeapCreate(sizeof(struct membership), MEMBER_HEAP_SIZE);
+	channel_heap = rb_bh_create(sizeof(struct Channel), CHANNEL_HEAP_SIZE, "channel_heap");
+	ban_heap = rb_bh_create(sizeof(struct Ban), BAN_HEAP_SIZE, "ban_heap");
+	topic_heap = rb_bh_create(sizeof(struct topic_info), TOPIC_HEAP_SIZE, "topic_heap");
+	member_heap = rb_bh_create(sizeof(struct membership), MEMBER_HEAP_SIZE, "member_heap");
 }
 
 /*
@@ -87,35 +85,35 @@ struct Channel *
 allocate_channel(const char *chname)
 {
 	struct Channel *chptr;
-	chptr = BlockHeapAlloc(channel_heap);
-	DupNString(chptr->chname, chname, CHANNELLEN);
-	return(chptr);
+	chptr = rb_bh_alloc(channel_heap);
+	chptr->chname = rb_strndup(chname, CHANNELLEN);
+	return (chptr);
 }
 
 void
 free_channel(struct Channel *chptr)
 {
-	MyFree(chptr->chname);
-	BlockHeapFree(channel_heap, chptr);
+	rb_free(chptr->chname);
+	rb_bh_free(channel_heap, chptr);
 }
 
 struct Ban *
 allocate_ban(const char *banstr, const char *who)
 {
 	struct Ban *bptr;
-	bptr = BlockHeapAlloc(ban_heap);
-	DupNString(bptr->banstr, banstr, BANLEN);
-	DupNString(bptr->who, who, BANLEN);
-	
-	return(bptr);
+	bptr = rb_bh_alloc(ban_heap);
+	bptr->banstr = rb_strndup(banstr, BANLEN);
+	bptr->who = rb_strndup(who, BANLEN);
+
+	return (bptr);
 }
 
 void
 free_ban(struct Ban *bptr)
 {
-	MyFree(bptr->banstr);
-	MyFree(bptr->who);
-	BlockHeapFree(ban_heap, bptr);
+	rb_free(bptr->banstr);
+	rb_free(bptr->who);
+	rb_bh_free(ban_heap, bptr);
 }
 
 
@@ -129,7 +127,7 @@ struct membership *
 find_channel_membership(struct Channel *chptr, struct Client *client_p)
 {
 	struct membership *msptr;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
 	if(!IsClient(client_p))
 		return NULL;
@@ -137,9 +135,9 @@ find_channel_membership(struct Channel *chptr, struct Client *client_p)
 	/* Pick the most efficient list to use to be nice to things like
 	 * CHANSERV which could be in a large number of channels
 	 */
-	if(dlink_list_length(&chptr->members) < dlink_list_length(&client_p->user->channel))
+	if(rb_dlink_list_length(&chptr->members) < rb_dlink_list_length(&client_p->user->channel))
 	{
-		DLINK_FOREACH(ptr, chptr->members.head)
+		RB_DLINK_FOREACH(ptr, chptr->members.head)
 		{
 			msptr = ptr->data;
 
@@ -149,7 +147,7 @@ find_channel_membership(struct Channel *chptr, struct Client *client_p)
 	}
 	else
 	{
-		DLINK_FOREACH(ptr, client_p->user->channel.head)
+		RB_DLINK_FOREACH(ptr, client_p->user->channel.head)
 		{
 			msptr = ptr->data;
 
@@ -204,17 +202,17 @@ add_user_to_channel(struct Channel *chptr, struct Client *client_p, int flags)
 	if(client_p->user == NULL)
 		return;
 
-	msptr = BlockHeapAlloc(member_heap);
+	msptr = rb_bh_alloc(member_heap);
 
 	msptr->chptr = chptr;
 	msptr->client_p = client_p;
 	msptr->flags = flags;
 
-	dlinkAdd(msptr, &msptr->usernode, &client_p->user->channel);
-	dlinkAdd(msptr, &msptr->channode, &chptr->members);
+	rb_dlinkAdd(msptr, &msptr->usernode, &client_p->user->channel);
+	rb_dlinkAdd(msptr, &msptr->channode, &chptr->members);
 
 	if(MyClient(client_p))
-		dlinkAdd(msptr, &msptr->locchannode, &chptr->locmembers);
+		rb_dlinkAdd(msptr, &msptr->locchannode, &chptr->locmembers);
 }
 
 /* remove_user_from_channel()
@@ -235,16 +233,16 @@ remove_user_from_channel(struct membership *msptr)
 	client_p = msptr->client_p;
 	chptr = msptr->chptr;
 
-	dlinkDelete(&msptr->usernode, &client_p->user->channel);
-	dlinkDelete(&msptr->channode, &chptr->members);
+	rb_dlinkDelete(&msptr->usernode, &client_p->user->channel);
+	rb_dlinkDelete(&msptr->channode, &chptr->members);
 
 	if(client_p->servptr == &me)
-		dlinkDelete(&msptr->locchannode, &chptr->locmembers);
+		rb_dlinkDelete(&msptr->locchannode, &chptr->locmembers);
 
-	if(dlink_list_length(&chptr->members) <= 0)
+	if(rb_dlink_list_length(&chptr->members) <= 0)
 		destroy_channel(chptr);
 
-	BlockHeapFree(member_heap, msptr);
+	rb_bh_free(member_heap, msptr);
 
 	return;
 }
@@ -260,26 +258,26 @@ remove_user_from_channels(struct Client *client_p)
 {
 	struct Channel *chptr;
 	struct membership *msptr;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 
 	if(client_p == NULL)
 		return;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->user->channel.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->user->channel.head)
 	{
 		msptr = ptr->data;
 		chptr = msptr->chptr;
 
-		dlinkDelete(&msptr->channode, &chptr->members);
+		rb_dlinkDelete(&msptr->channode, &chptr->members);
 
 		if(client_p->servptr == &me)
-			dlinkDelete(&msptr->locchannode, &chptr->locmembers);
+			rb_dlinkDelete(&msptr->locchannode, &chptr->locmembers);
 
-		if(dlink_list_length(&chptr->members) <= 0)
+		if(rb_dlink_list_length(&chptr->members) <= 0)
 			destroy_channel(chptr);
 
-		BlockHeapFree(member_heap, msptr);
+		rb_bh_free(member_heap, msptr);
 	}
 
 	client_p->user->channel.head = client_p->user->channel.tail = NULL;
@@ -297,15 +295,15 @@ void
 invalidate_bancache_user(struct Client *client_p)
 {
 	struct membership *msptr;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
 	if(client_p == NULL)
 		return;
 
-	DLINK_FOREACH(ptr, client_p->user->channel.head)
+	RB_DLINK_FOREACH(ptr, client_p->user->channel.head)
 	{
 		msptr = ptr->data;
-		msptr->bants = 0;
+		msptr->ban_serial = 0;
 		msptr->flags &= ~CHFL_BANNED;
 	}
 }
@@ -323,7 +321,7 @@ check_channel_name(const char *name)
 	if(name == NULL)
 		return 0;
 
-	for (; *name; ++name)
+	for(; *name; ++name)
 	{
 		if(!IsChanChar(*name))
 			return 0;
@@ -339,13 +337,13 @@ check_channel_name(const char *name)
  * side effects - list of b/e/I modes is cleared
  */
 void
-free_channel_list(dlink_list *list)
+free_channel_list(rb_dlink_list *list)
 {
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 	struct Ban *actualBan;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
 	{
 		actualBan = ptr->data;
 		free_ban(actualBan);
@@ -364,9 +362,9 @@ free_channel_list(dlink_list *list)
 void
 destroy_channel(struct Channel *chptr)
 {
-	dlink_node *ptr, *next_ptr;
+	rb_dlink_node *ptr, *next_ptr;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, chptr->invites.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, chptr->invites.head)
 	{
 		del_invite(chptr, ptr->data);
 	}
@@ -379,8 +377,8 @@ destroy_channel(struct Channel *chptr)
 	/* Free the topic */
 	free_topic(chptr);
 
-	dlinkDelete(&chptr->node, &global_channel_list);
-	del_from_channel_hash(chptr->chname, chptr);
+	rb_dlinkDelete(&chptr->node, &global_channel_list);
+	del_from_hash(HASH_CHANNEL, chptr->chname, chptr);
 	free_channel(chptr);
 }
 
@@ -411,7 +409,7 @@ channel_member_names(struct Channel *chptr, struct Client *client_p, int show_eo
 {
 	struct membership *msptr;
 	struct Client *target_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	char lbuf[BUFSIZE];
 	char *t;
 	int mlen;
@@ -419,19 +417,18 @@ channel_member_names(struct Channel *chptr, struct Client *client_p, int show_eo
 	int cur_len;
 	int is_member;
 	int stack = IsCapable(client_p, CLICAP_MULTI_PREFIX);
-
+	SetCork(client_p);
 	if(ShowChannel(client_p, chptr))
 	{
 		is_member = IsMember(client_p, chptr);
 
-		cur_len = mlen = ircsprintf(lbuf, form_str(RPL_NAMREPLY),
-					    me.name, client_p->name, 
-					    channel_pub_or_secret(chptr),
-					    chptr->chname);
+		cur_len = mlen = rb_sprintf(lbuf, form_str(RPL_NAMREPLY),
+					    me.name, client_p->name,
+					    channel_pub_or_secret(chptr), chptr->chname);
 
 		t = lbuf + cur_len;
 
-		DLINK_FOREACH(ptr, chptr->members.head)
+		RB_DLINK_FOREACH(ptr, chptr->members.head)
 		{
 			msptr = ptr->data;
 			target_p = msptr->client_p;
@@ -440,16 +437,16 @@ channel_member_names(struct Channel *chptr, struct Client *client_p, int show_eo
 				continue;
 
 			/* space, possible "@+" prefix */
-			if(cur_len + strlen(target_p->name) + 3 >= BUFSIZE-3)
+			if(cur_len + strlen(target_p->name) + 3 >= BUFSIZE - 3)
 			{
 				*(t - 1) = '\0';
-				sendto_one(client_p, "%s", lbuf);
+				sendto_one_buffer(client_p, lbuf);
 				cur_len = mlen;
 				t = lbuf + mlen;
 			}
 
-			tlen = ircsprintf(t, "%s%s ", find_channel_status(msptr, stack),
-				   target_p->name);
+			tlen = rb_sprintf(t, "%s%s ", find_channel_status(msptr, stack),
+					  target_p->name);
 
 			cur_len += tlen;
 			t += tlen;
@@ -465,13 +462,15 @@ channel_member_names(struct Channel *chptr, struct Client *client_p, int show_eo
 		if(cur_len != mlen)
 		{
 			*(t - 1) = '\0';
-			sendto_one(client_p, "%s", lbuf);
+			sendto_one_buffer(client_p, lbuf);
 		}
 	}
 
 	if(show_eon)
 		sendto_one(client_p, form_str(RPL_ENDOFNAMES),
 			   me.name, client_p->name, chptr->chname);
+	ClearCork(client_p);
+	send_pop_queue(client_p);
 }
 
 /* del_invite()
@@ -483,8 +482,8 @@ channel_member_names(struct Channel *chptr, struct Client *client_p, int show_eo
 void
 del_invite(struct Channel *chptr, struct Client *who)
 {
-	dlinkFindDestroy(who, &chptr->invites);
-	dlinkFindDestroy(chptr, &who->localClient->invited);
+	rb_dlinkFindDestroy(who, &chptr->invites);
+	rb_dlinkFindDestroy(chptr, &who->localClient->invited);
 }
 
 /* is_banned()
@@ -500,7 +499,7 @@ is_banned(struct Channel *chptr, struct Client *who, struct membership *msptr,
 {
 	char src_host[NICKLEN + USERLEN + HOSTLEN + 6];
 	char src_iphost[NICKLEN + USERLEN + HOSTLEN + 6];
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	struct Ban *actualBan = NULL;
 	struct Ban *actualExcept = NULL;
 
@@ -510,21 +509,18 @@ is_banned(struct Channel *chptr, struct Client *who, struct membership *msptr,
 	/* if the buffers havent been built, do it here */
 	if(s == NULL)
 	{
-		ircsprintf(src_host, "%s!%s@%s",
-			   who->name, who->username, who->host);
-		ircsprintf(src_iphost, "%s!%s@%s",
-			   who->name, who->username, who->sockhost);
+		rb_sprintf(src_host, "%s!%s@%s", who->name, who->username, who->host);
+		rb_sprintf(src_iphost, "%s!%s@%s", who->name, who->username, who->sockhost);
 
 		s = src_host;
 		s2 = src_iphost;
 	}
 
-	DLINK_FOREACH(ptr, chptr->banlist.head)
+	RB_DLINK_FOREACH(ptr, chptr->banlist.head)
 	{
 		actualBan = ptr->data;
 		if(match(actualBan->banstr, s) ||
-		   match(actualBan->banstr, s2) || 
-		   match_cidr(actualBan->banstr, s2))
+		   match(actualBan->banstr, s2) || match_cidr(actualBan->banstr, s2))
 			break;
 		else
 			actualBan = NULL;
@@ -532,19 +528,18 @@ is_banned(struct Channel *chptr, struct Client *who, struct membership *msptr,
 
 	if((actualBan != NULL) && ConfigChannel.use_except)
 	{
-		DLINK_FOREACH(ptr, chptr->exceptlist.head)
+		RB_DLINK_FOREACH(ptr, chptr->exceptlist.head)
 		{
 			actualExcept = ptr->data;
 
 			/* theyre exempted.. */
 			if(match(actualExcept->banstr, s) ||
-			   match(actualExcept->banstr, s2) || 
-			   match_cidr(actualExcept->banstr, s2))
+			   match(actualExcept->banstr, s2) || match_cidr(actualExcept->banstr, s2))
 			{
 				/* cache the fact theyre not banned */
 				if(msptr != NULL)
 				{
-					msptr->bants = chptr->bants;
+					msptr->ban_serial = chptr->ban_serial;
 					msptr->flags &= ~CHFL_BANNED;
 				}
 
@@ -556,7 +551,7 @@ is_banned(struct Channel *chptr, struct Client *who, struct membership *msptr,
 	/* cache the banned/not banned status */
 	if(msptr != NULL)
 	{
-		msptr->bants = chptr->bants;
+		msptr->ban_serial = chptr->ban_serial;
 
 		if(actualBan != NULL)
 		{
@@ -573,70 +568,6 @@ is_banned(struct Channel *chptr, struct Client *who, struct membership *msptr,
 	return ((actualBan ? CHFL_BAN : 0));
 }
 
-/* can_join()
- *
- * input	- client to check, channel to check for, key
- * output	- reason for not being able to join, else 0
- * side effects -
- */
-int
-can_join(struct Client *source_p, struct Channel *chptr, char *key)
-{
-	dlink_node *lp;
-	dlink_node *ptr;
-	struct Ban *invex = NULL;
-	char src_host[NICKLEN + USERLEN + HOSTLEN + 6];
-	char src_iphost[NICKLEN + USERLEN + HOSTLEN + 6];
-
-	s_assert(source_p->localClient != NULL);
-
-	ircsprintf(src_host, "%s!%s@%s", 
-		   source_p->name, source_p->username, source_p->host);
-	ircsprintf(src_iphost, "%s!%s@%s",
-		   source_p->name, source_p->username, source_p->sockhost);
-
-	if((is_banned(chptr, source_p, NULL, src_host, src_iphost)) == CHFL_BAN)
-		return (ERR_BANNEDFROMCHAN);
-
-	if(chptr->mode.mode & MODE_INVITEONLY)
-	{
-		DLINK_FOREACH(lp, source_p->localClient->invited.head)
-		{
-			if(lp->data == chptr)
-				break;
-		}
-		if(lp == NULL)
-		{
-			if(!ConfigChannel.use_invex)
-				return (ERR_INVITEONLYCHAN);
-			DLINK_FOREACH(ptr, chptr->invexlist.head)
-			{
-				invex = ptr->data;
-				if(match(invex->banstr, src_host)
-				   || match(invex->banstr, src_iphost)
-				   || match_cidr(invex->banstr, src_iphost))
-					break;
-			}
-			if(ptr == NULL)
-				return (ERR_INVITEONLYCHAN);
-		}
-	}
-
-	if(*chptr->mode.key && (EmptyString(key) || irccmp(chptr->mode.key, key)))
-		return (ERR_BADCHANNELKEY);
-
-	if(chptr->mode.limit && 
-	   dlink_list_length(&chptr->members) >= (unsigned long)chptr->mode.limit)
-		return (ERR_CHANNELISFULL);
-
-#ifdef ENABLE_SERVICES
-	if(chptr->mode.mode & MODE_REGONLY && EmptyString(source_p->user->suser))
-		return ERR_NEEDREGGEDNICK;
-#endif
-
-	return 0;
-}
-
 /* can_send()
  *
  * input	- user to check in channel, membership pointer
@@ -644,8 +575,7 @@ can_join(struct Client *source_p, struct Channel *chptr, char *key)
  * side effects -
  */
 int
-can_send(struct Channel *chptr, struct Client *source_p, 
-	 struct membership *msptr)
+can_send(struct Channel *chptr, struct Client *source_p, struct membership *msptr)
 {
 	if(IsServer(source_p))
 		return CAN_SEND_OPV;
@@ -664,8 +594,7 @@ can_send(struct Channel *chptr, struct Client *source_p,
 			 * they cant send.  we dont check bans here because
 			 * theres no possibility of caching them --fl
 			 */
-			if(chptr->mode.mode & MODE_NOPRIVMSGS ||
-			   chptr->mode.mode & MODE_MODERATED)
+			if(chptr->mode.mode & MODE_NOPRIVMSGS || chptr->mode.mode & MODE_MODERATED)
 				return CAN_SEND_NO;
 			else
 				return CAN_SEND_NONOP;
@@ -681,7 +610,7 @@ can_send(struct Channel *chptr, struct Client *source_p,
 	if(ConfigChannel.quiet_on_ban && MyClient(source_p))
 	{
 		/* cached can_send */
-		if(msptr->bants == chptr->bants)
+		if(msptr->ban_serial == chptr->ban_serial)
 		{
 			if(can_send_banned(msptr))
 				return CAN_SEND_NO;
@@ -731,7 +660,7 @@ check_spambot_warning(struct Client *source_p, const char *name)
 	else
 	{
 		if((t_delta =
-		    (CurrentTime - source_p->localClient->last_leave_time)) >
+		    (rb_current_time() - source_p->localClient->last_leave_time)) >
 		   JOIN_LEAVE_COUNT_EXPIRE_TIME)
 		{
 			decrement_count = (t_delta / JOIN_LEAVE_COUNT_EXPIRE_TIME);
@@ -742,7 +671,7 @@ check_spambot_warning(struct Client *source_p, const char *name)
 		}
 		else
 		{
-			if((CurrentTime -
+			if((rb_current_time() -
 			    (source_p->localClient->last_join_time)) < GlobalSetOptions.spam_time)
 			{
 				/* oh, its a possible spambot */
@@ -750,9 +679,9 @@ check_spambot_warning(struct Client *source_p, const char *name)
 			}
 		}
 		if(name != NULL)
-			source_p->localClient->last_join_time = CurrentTime;
+			source_p->localClient->last_join_time = rb_current_time();
 		else
-			source_p->localClient->last_leave_time = CurrentTime;
+			source_p->localClient->last_leave_time = rb_current_time();
 	}
 }
 
@@ -777,8 +706,10 @@ check_splitmode(void *unused)
 			{
 				splitmode = 1;
 				sendto_realops_flags(UMODE_ALL, L_ALL,
-					     "Network split, activating splitmode");
-				eventAddIsh("check_splitmode", check_splitmode, NULL, 2);
+						     "Network split, activating splitmode");
+				checksplit_ev =
+					rb_event_addish("check_splitmode", check_splitmode, NULL,
+							5);
 			}
 		}
 		/* in splitmode, check whether its finished */
@@ -787,9 +718,10 @@ check_splitmode(void *unused)
 			splitmode = 0;
 
 			sendto_realops_flags(UMODE_ALL, L_ALL,
-				     "Network rejoined, deactivating splitmode");
+					     "Network rejoined, deactivating splitmode");
 
-			eventDelete(check_splitmode, NULL);
+			rb_event_delete(checksplit_ev);
+			checksplit_ev = NULL;
 		}
 	}
 }
@@ -804,21 +736,10 @@ check_splitmode(void *unused)
 static void
 allocate_topic(struct Channel *chptr)
 {
-	void *ptr;
-
 	if(chptr == NULL)
 		return;
 
-	ptr = BlockHeapAlloc(topic_heap);
-
-	/* Basically we allocate one large block for the topic and
-	 * the topic info.  We then split it up into two and shove it
-	 * in the chptr 
-	 */
-	chptr->topic = ptr;
-	chptr->topic_info = (char *) ptr + TOPICLEN + 1;
-	*chptr->topic = '\0';
-	*chptr->topic_info = '\0';
+	chptr->topic = rb_bh_alloc(topic_heap);
 }
 
 /* free_topic()
@@ -830,18 +751,15 @@ allocate_topic(struct Channel *chptr)
 static void
 free_topic(struct Channel *chptr)
 {
-	void *ptr;
-
 	if(chptr == NULL || chptr->topic == NULL)
 		return;
 
 	/* This is safe for now - If you change allocate_topic you
 	 * MUST change this as well
 	 */
-	ptr = chptr->topic;
-	BlockHeapFree(topic_heap, ptr);
+	rb_free(chptr->topic->topic);
+	rb_bh_free(topic_heap, chptr->topic);
 	chptr->topic = NULL;
-	chptr->topic_info = NULL;
 }
 
 /* set_channel_topic()
@@ -851,22 +769,23 @@ free_topic(struct Channel *chptr)
  * side effects - channels topic, topic info and TS are set.
  */
 void
-set_channel_topic(struct Channel *chptr, const char *topic,
-		  const char *topic_info, time_t topicts)
+set_channel_topic(struct Channel *chptr, const char *topic, const char *topic_info, time_t topicts)
 {
 	if(strlen(topic) > 0)
 	{
 		if(chptr->topic == NULL)
 			allocate_topic(chptr);
-		strlcpy(chptr->topic, topic, TOPICLEN + 1);
-		strlcpy(chptr->topic_info, topic_info, USERHOST_REPLYLEN);
-		chptr->topic_time = topicts;
+		else
+			rb_free(chptr->topic->topic);
+
+		chptr->topic->topic = rb_strndup(topic, ConfigChannel.topiclen + 1);	/* the + 1 for the \0 */
+		rb_strlcpy(chptr->topic->topic_info, topic_info, sizeof(chptr->topic->topic_info));
+		chptr->topic->topic_time = topicts;
 	}
 	else
 	{
 		if(chptr->topic != NULL)
 			free_topic(chptr);
-		chptr->topic_time = 0;
 	}
 }
 
@@ -901,26 +820,29 @@ channel_modes(struct Channel *chptr, struct Client *client_p)
 	if(chptr->mode.mode & MODE_REGONLY)
 		*mbuf++ = 'r';
 #endif
+	if(chptr->mode.mode & MODE_SSLONLY)
+		*mbuf++ = 'S';
+
 	if(chptr->mode.limit && *chptr->mode.key)
 	{
 		if(IsMe(client_p) || !MyClient(client_p) || IsMember(client_p, chptr))
-			ircsprintf(mbuf, "lk %d %s", chptr->mode.limit, chptr->mode.key);
+			rb_sprintf(mbuf, "lk %d %s", chptr->mode.limit, chptr->mode.key);
 		else
-			ircsprintf(mbuf, "lk");
+			strcpy(mbuf, "lk");
 	}
 	else if(chptr->mode.limit)
 	{
 		if(IsMe(client_p) || !MyClient(client_p) || IsMember(client_p, chptr))
-			ircsprintf(mbuf, "l %d", chptr->mode.limit);
+			rb_sprintf(mbuf, "l %d", chptr->mode.limit);
 		else
-			ircsprintf(mbuf, "l");
+			strcpy(mbuf, "l");
 	}
 	else if(*chptr->mode.key)
 	{
 		if(IsMe(client_p) || !MyClient(client_p) || IsMember(client_p, chptr))
-			ircsprintf(mbuf, "k %s", chptr->mode.key);
+			rb_sprintf(mbuf, "k %s", chptr->mode.key);
 		else
-			ircsprintf(mbuf, "k");
+			strcpy(mbuf, "k");
 	}
 	else
 		*mbuf = '\0';
@@ -954,10 +876,10 @@ init_chcap_usage_counts(void)
 	memset(chcap_combos, 0, sizeof(chcap_combos));
 
 	/* For every possible combination */
-	for (m = 0; m < NCHCAP_COMBOS; m++)
+	for(m = 0; m < NCHCAP_COMBOS; m++)
 	{
 		/* Check each capab */
-		for (c = y = n = 0; c < NCHCAPS; c++)
+		for(c = y = n = 0; c < NCHCAPS; c++)
 		{
 			if((m & (1 << c)) == 0)
 				n |= channel_capabs[c];
@@ -980,7 +902,7 @@ set_chcap_usage_counts(struct Client *serv_p)
 {
 	int n;
 
-	for (n = 0; n < NCHCAP_COMBOS; n++)
+	for(n = 0; n < NCHCAP_COMBOS; n++)
 	{
 		if(IsCapable(serv_p, chcap_combos[n].cap_yes) &&
 		   NotCapable(serv_p, chcap_combos[n].cap_no))
@@ -1006,7 +928,7 @@ unset_chcap_usage_counts(struct Client *serv_p)
 {
 	int n;
 
-	for (n = 0; n < NCHCAP_COMBOS; n++)
+	for(n = 0; n < NCHCAP_COMBOS; n++)
 	{
 		if(IsCapable(serv_p, chcap_combos[n].cap_yes) &&
 		   NotCapable(serv_p, chcap_combos[n].cap_no))
@@ -1038,8 +960,7 @@ unset_chcap_usage_counts(struct Client *serv_p)
  */
 void
 send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
-		      struct Channel *chptr, struct ChModeChange mode_changes[],
-		      int mode_count)
+		      struct Channel *chptr, struct ChModeChange mode_changes[], int mode_count)
 {
 	static char modebuf[BUFSIZE];
 	static char parabuf[BUFSIZE];
@@ -1053,7 +974,7 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
 	int arglen;
 
 	/* Now send to servers... */
-	for (j = 0; j < NCHCAP_COMBOS; j++)
+	for(j = 0; j < NCHCAP_COMBOS; j++)
 	{
 		if(chcap_combos[j].count == 0)
 			continue;
@@ -1069,23 +990,23 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
 		nocap = chcap_combos[j].cap_no;
 
 		if(cap & CAP_TS6)
-			mbl = preflen = ircsprintf(modebuf, ":%s TMODE %ld %s ",
-						use_id(source_p), (long) chptr->channelts,
-						chptr->chname);
+			mbl = preflen = rb_sprintf(modebuf, ":%s TMODE %ld %s ",
+						   use_id(source_p), (long)chptr->channelts,
+						   chptr->chname);
 		else
-			mbl = preflen = ircsprintf(modebuf, ":%s MODE %s ",
-						source_p->name, chptr->chname);
+			mbl = preflen = rb_sprintf(modebuf, ":%s MODE %s ",
+						   source_p->name, chptr->chname);
 
 		/* loop the list of - modes we have */
-		for (i = 0; i < mode_count; i++)
+		for(i = 0; i < mode_count; i++)
 		{
 			/* if they dont support the cap we need, or they do support a cap they
 			 * cant have, then dont add it to the modebuf.. that way they wont see
 			 * the mode
 			 */
 			if((mode_changes[i].letter == 0) ||
-				((cap & mode_changes[i].caps) != mode_changes[i].caps)
-				|| ((nocap & mode_changes[i].nocaps) != mode_changes[i].nocaps))
+			   ((cap & mode_changes[i].caps) != mode_changes[i].caps)
+			   || ((nocap & mode_changes[i].nocaps) != mode_changes[i].nocaps))
 				continue;
 
 			if((cap & CAP_TS6) && !EmptyString(mode_changes[i].id))
@@ -1098,7 +1019,7 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
 				arglen = strlen(arg);
 
 				/* dont even think about it! --fl */
-				if(arglen > MODEBUFLEN-5)
+				if(arglen > MODEBUFLEN - 5)
 					continue;
 			}
 
@@ -1110,11 +1031,11 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
 			 * which even then won't work as we don't always know the uid -A1kmm.
 			 */
 			if(arg && ((mc == MAXMODEPARAMSSERV) ||
-				((mbl + pbl + arglen + 4) > (BUFSIZE-3))))
+				   ((mbl + pbl + arglen + 4) > (BUFSIZE - 3))))
 			{
 				if(nc != 0)
 					sendto_server(client_p, chptr, cap, nocap,
-							"%s %s", modebuf, parabuf);
+						      "%s %s", modebuf, parabuf);
 				nc = 0;
 				mc = 0;
 
@@ -1137,7 +1058,7 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
 
 			if(arg != NULL)
 			{
-				len = ircsprintf(pbuf, "%s ", arg);
+				len = rb_sprintf(pbuf, "%s ", arg);
 				pbuf += len;
 				pbl += len;
 				mc++;
