@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
  *  Copyright (C) 1996-2002 Hybrid Development Team
- *  Copyright (C) 2002-2005 ircd-ratbox development team
+ *  Copyright (C) 2002-2012 ircd-ratbox development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: ircd.c 26304 2008-12-16 03:08:43Z androsyn $
+ *  $Id: ircd.c 27399 2012-05-27 13:09:42Z jilles $
  */
 
 #include "ratbox_lib.h"
@@ -567,7 +567,7 @@ ratbox_main(int argc, char *argv[])
 	if(r > 0)
 	{
 		fprintf(stderr,
-			"ERROR: Unable to start ircd: found %d syntax error(s) whlist loading config file.  Try running with -conftest\n",
+			"ERROR: Unable to start ircd: found %d syntax error(s) whilst loading config file.  Try running with -conftest\n",
 			r);
 		return 1;
 	}
