@@ -1,7 +1,7 @@
 /*
  *  sslproc.c: An interface to ssld
  *  Copyright (C) 2007 Aaron Sethman <androsyn@ratbox.org>
- *  Copyright (C) 2007 ircd-ratbox development team
+ *  Copyright (C) 2007-2012 ircd-ratbox development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: sslproc.c 26517 2009-05-13 15:10:29Z androsyn $
+ *  $Id: sslproc.c 27371 2012-03-16 05:33:15Z dubkat $
  */
 
 #include <ratbox_lib.h>
@@ -434,13 +434,13 @@ ssl_process_cmd_recv(ssl_ctl_t * ctl)
 			break;
 		case 'I':
 			ircd_ssl_ok = 0;
-			ilog(L_MAIN, cannot_setup_ssl);
-			sendto_realops_flags(UMODE_ALL, L_ALL, cannot_setup_ssl);
+			ilog(L_MAIN, "%s", cannot_setup_ssl);
+			sendto_realops_flags(UMODE_ALL, L_ALL, "%s", cannot_setup_ssl);
 		case 'U':
 			zlib_ok = 0;
 			ircd_ssl_ok = 0;
-			ilog(L_MAIN, no_ssl_or_zlib);
-			sendto_realops_flags(UMODE_ALL, L_ALL, no_ssl_or_zlib);
+			ilog(L_MAIN, "%s", no_ssl_or_zlib);
+			sendto_realops_flags(UMODE_ALL, L_ALL, "%s", no_ssl_or_zlib);
 			ssl_killall();
 			break;
 		case 'z':
